//---------------------------------------------------------------------------
#ifndef CWavRingBufferH
#define CWavRingBufferH

#include <iostream>
#include <cstddef>
#include <cstring>
#include <array>
#include <vector>
using namespace std;

//---------------------------------------------------------------------------
class CWavRingBuffer
{
private:
	unsigned long DefaultCBuffSize;

	unsigned int	m_BufferCount;	// バッファ容量(要素数)
	bool			m_bPrepared;	// 
	bool m_bFull;					// バッファが満タンか否かで、読み出し開始点の処理が変わる

	std::vector<float> m_Buffer;	// バッファ本体
	
	std::vector<float>::iterator	m_CurrentReadPoint;	// 読み出し開始点
	std::vector<float>::iterator	m_CurrentWritePoint;// 書き出し開始点

	std::vector<float>::iterator 	m_head;	// バッファの先頭(m_buffer.begin())
	std::vector<float>::iterator 	m_tail; // バッファの末尾(m_buffer.end())

public:
	
	CWavRingBuffer()
	{
		DefaultCBuffSize = 60*48000*8;
		m_Buffer.resize(DefaultCBuffSize);
  		m_BufferCount = 0;
  		m_head = m_Buffer.begin();
  		m_tail = m_Buffer.end();

		m_CurrentReadPoint = m_head;
		m_CurrentWritePoint = m_head;

		m_Buffer.assign(0,m_Buffer.size());
		m_bPrepared = false;
		m_bFull = false;

	}

	virtual ~CWavRingBuffer()
	{
		this->clear();
	}

std::vector<float>::iterator begin(){
	return m_CurrentReadPoint;
}

std::vector<float>::iterator end(){
	return m_CurrentWritePoint;
}

std::vector<float>::iterator head(){
	return m_head;
}

std::vector<float>::iterator tail(){
	return m_tail;
}



//---------------------------------------------------------------------------

void ClearParameters()
{
	m_CurrentReadPoint = m_head;
    m_CurrentWritePoint = m_head;
    m_bFull = false;

    if (&m_Buffer[0]){
        m_Buffer.assign(m_Buffer.size(),0);
    }

}

void clear(){
	this->ClearParameters();
}

//---------------------------------------------------------------------------

// イテレータtarget が ref　よりもインデックス大なら正、小なら負
int itr_comp(std::vector<float>::iterator target,
	  std::vector<float>::iterator ref){

	if(target > ref){
		return 1;
	}else if(target == ref){
		return 0;
	}else{
		return -1;
	}
}


//---------------------------------------------------------------------------

//　バッファにデータを格納するための初期化 Prepare -> copy -> insert の順
unsigned int Prepare(unsigned int dwDataCount)
{

	if (dwDataCount == 0) {
		return(0);
	}
	//	内部変数を初期化
	this->clear();

	//	バッファの確保
	m_Buffer.clear();
	m_Buffer.resize(dwDataCount);
	m_Buffer.reserve(dwDataCount);
	if (!(&m_Buffer[0])) {
		return((unsigned int)-1);
	}

	if (&m_Buffer[0]){
		m_Buffer.assign(m_Buffer.size(),0);
	}

	m_head = m_Buffer.begin();
	m_tail = m_Buffer.end();

	m_CurrentReadPoint = m_head;
	m_CurrentWritePoint = m_head;	

	m_bPrepared = true;
	m_bFull = false;
	m_BufferCount = dwDataCount;


	return(dwDataCount);
}
//---------------------------------------------------------------------------

//　From...Toの内部バッファのデータを　Destから始まる外部データに書き出し
void copy(
		  std::vector<float>::iterator From,
		  std::vector<float>::iterator To,
		  std::vector<short>::iterator Dest)
{

	// 指定されたイテレータがバッファ内にあるようにする。
	if(distance(m_head, From) > distance(m_head, m_tail)){
		From -= m_BufferCount;
	}
	if(distance(m_head, To) > distance(m_head, m_tail)){
		To -= m_BufferCount;
	}

	// From, Toの位置関係に応じて、読み込む要素数を変更
	if(From == To){
		this->Read(Dest, From, m_BufferCount);
	}else if(From > To){
		this->Read(Dest, From, m_BufferCount - (unsigned int)std::distance(To,From)+1);			
	}else{
		this->Read(Dest, From, (unsigned int)std::distance(From,To));
	}
	return;
}


// copy の本体
unsigned int Read(	std::vector<short>::iterator pData,// 出力先
					std::vector<float>::iterator itRead,// 内部バッファの開始位置
					unsigned int dwDataCount)			// 書き出す要素数
{

	unsigned int	i;
	unsigned int	dwCopyCnt;

	if (/*pData == NULL || */dwDataCount == 0) {
		return(0);
	}
		//	受け入れ先を初期化

	// バッファサイズまでしか書かない
	if (m_BufferCount > dwDataCount){
		dwCopyCnt = dwDataCount;
	}
	else{
		dwCopyCnt = m_BufferCount;
	}

	// 書き込みバッファを追い越さないようにする
    unsigned int	dwReadable;
    if(itr_comp(itRead,m_CurrentWritePoint)>=0){
    	dwReadable = m_BufferCount - std::distance(m_CurrentWritePoint, itRead);
    }else{
    	dwReadable = std::distance(itRead, m_CurrentWritePoint);
    }

    if (dwReadable < dwCopyCnt){
        dwCopyCnt = dwReadable;
    }


	std::vector<float>::iterator ReadPoint = itRead; //読み出し位置指定イテレータ
		//	データを渡す処理
	std::vector<short>::iterator pcDst = pData;
	for (i = 0; i < dwCopyCnt; i++, pcDst++) {
		*pcDst = *ReadPoint;
		ReadPoint++;
		if (ReadPoint == m_tail) {
			ReadPoint = m_head;
			}
	}

	return(i);
}
//---------------------------------------------------------------------------
//　pData...itTo内の外部データを　内部バッファitDestへ読み込み
unsigned int insert(
			std::vector<float>::iterator itDest, 
			std::vector<short>::iterator pData, 
			std::vector<short>::iterator itTo)
{
	


	unsigned int dwDataCount = (unsigned int)std::distance(pData,itTo);//読み込み要素数
	unsigned int	i, dwCopyCnt;


	if (/*pData == NULL || */dwDataCount == 0) {
		return(0);
	}
	
	bool overrun = false;
	// バッファサイズより多いデータは末尾のみ読み込み
	if (m_BufferCount < dwDataCount){
		overrun = true;
		dwCopyCnt = m_BufferCount;
		pData = pData + dwDataCount - dwCopyCnt;
	}else{
		dwCopyCnt = dwDataCount;	
	}


	std::vector<float>::iterator WritePoint; //書き込み位置指定バッファ
	//バッファの要素に空きができないように調整
	if(distance(m_head,itDest) > m_Buffer.size()){
		WritePoint = m_CurrentWritePoint;
	}else{
		WritePoint = itDest;
	}

	// WritePointとm_CurrentWritePointの位置関係の変化を記録
	int comp_before = itr_comp(WritePoint, m_CurrentWritePoint);
	
	//	データを渡す処理
	std::vector<short>::iterator pcSrc = pData;
	for (i = 0; i < dwCopyCnt; i++, pcSrc++) {
		*WritePoint = *pcSrc;
		WritePoint++;
		if (WritePoint == m_tail) {
			WritePoint = m_head;
			m_bFull = true;
		}
	}


	// WritePointのオーバーラン処理
	int comp_after = itr_comp(WritePoint, m_CurrentWritePoint);
	if((comp_before <= 0 && comp_after >= 0) || (comp_before >= 0 && comp_after <= 0) ){
		m_CurrentWritePoint = WritePoint;
	}

	// ReadPoint のオーバーラン処理
	if(m_bFull){
		m_CurrentReadPoint = m_CurrentWritePoint;
		if (m_CurrentReadPoint == m_tail) {
			m_CurrentReadPoint = m_head;
			
		}
	}

	return(i);
}
//---------------------------------------------------------------------------

unsigned int advance(
			std::vector<float>::iterator& itr, 
			unsigned int advanceCount)
{
	if (itr == m_tail) {
		itr = m_head;
	}
	// int szBuf = distance(m_head,m_tail);

	int adv = (int)advanceCount % (int)m_BufferCount;
	//if(adv < 0){
 
	if(distance(m_head,itr) + adv >= m_BufferCount ){
			adv -= m_BufferCount;
	}
 
	// itr = m_head + (distance(m_head,itr) + adv);
	itr += adv;

	if (itr == m_tail) {
		itr = m_head;
	}

	if (itr >= m_tail) {
	}

	if (itr < m_head){

		itr += m_BufferCount;

	}



	return 0;
}


bool IsPrepared()
{
	return(this->m_bPrepared);
}
//---------------------------------------------------------------------------


unsigned int ReadCnt(std::vector<float>::iterator read)
{
		return (distance( m_head,m_CurrentReadPoint));
}
//---------------------------------------------------------------------------

unsigned int WriteCnt(void)
{
	return (m_BufferCount + distance( m_head,m_CurrentWritePoint));
}
//---------------------------------------------------------------------------


unsigned int set_capacity(unsigned int dwDataCount){
	this->Prepare(dwDataCount);
	return dwDataCount;
}

size_t size(){
	if(!m_bFull){
		return (size_t)distance(m_head, m_CurrentWritePoint);	
	}else{
		return m_Buffer.size(); 
	}
}



//-----------------------------------------------------------------------
//-----------------------------------------------------------------------
//関数オーバーロード引数がfloatのイテレータの場合
void copy(
		  std::vector<float>::iterator From,
		  std::vector<float>::iterator To,
		  std::vector<float>::iterator Dest)
{


	if(distance(m_head, From) > distance(m_head, m_tail)){
		From -= m_BufferCount;
	}
	if(distance(m_head, To) > distance(m_head, m_tail)){
		To -= m_BufferCount;
	}

	if(From == To){
		this->Read(Dest, From, m_BufferCount);
	}else if(From > To){
		this->Read(Dest, From, m_BufferCount - (unsigned int)std::distance(To,From)+1);			
	}else{
		this->Read(Dest, From, (unsigned int)std::distance(From,To));
	}
	return;
}


//
unsigned int Read(	std::vector<float>::iterator pData,
					std::vector<float>::iterator itRead,
					unsigned int dwDataCount)
{

	unsigned int	i;
	unsigned int	dwCopyCnt;

	if (/*pData == NULL || */dwDataCount == 0) {
		return(0);
	}
		//	受け入れ先を初期化
//	memset(pData, 0, dwDataCount);

	// バッファサイズまでしか書かない
	if (m_BufferCount > dwDataCount){
		dwCopyCnt = dwDataCount;
	}
	else{
		dwCopyCnt = m_BufferCount;
	}


    unsigned int	dwReadable;
    if(itRead == m_CurrentWritePoint){
    	dwReadable = m_BufferCount;
    }else if(itr_comp(itRead,m_CurrentWritePoint)>=0){
    	dwReadable = m_BufferCount - std::distance(m_CurrentWritePoint, itRead);
    }else{
    	dwReadable = std::distance(itRead, m_CurrentWritePoint);
    }


    // 書き込みバッファを追い越さないようにする
    if (dwReadable < dwCopyCnt){
        dwCopyCnt = dwReadable;
    }
	
	std::vector<float>::iterator ReadPoint = itRead;	
		//	データを渡す処理
	std::vector<float>::iterator pcDst = pData;
	for (i = 0; i < dwCopyCnt; i++, pcDst++) {
		*pcDst = *ReadPoint;
		ReadPoint++;
		if (ReadPoint == m_tail) {
			ReadPoint = m_head;
			}
	}

	return(i);
}
//---------------------------------------------------------------------------
/*書き込み位置指定に対応させる*/
unsigned int insert(
			std::vector<float>::iterator itDest, 
			std::vector<float>::iterator pData, 
			std::vector<float>::iterator itTo)
{
	unsigned int dwDataCount = (unsigned int)std::distance(pData,itTo);
	unsigned int	i, dwCopyCnt;



	if (/*pData == NULL || */dwDataCount == 0) {
		return(0);
	}
	
	bool overrun = false;
	// バッファサイズより多いデータはmatsubi ni
	if (m_BufferCount < dwDataCount){
		overrun = true;
		dwCopyCnt = m_BufferCount;
		pData = pData + dwDataCount - dwCopyCnt;
	}else{
		dwCopyCnt = dwDataCount;	
	}


		//	データを渡す処理
	std::vector<float>::iterator WritePoint;
	if(distance(m_head,itDest) > m_Buffer.size()){
		WritePoint = m_CurrentWritePoint;
	}else{
		WritePoint = itDest;
	}

	int comp_before = itr_comp(WritePoint, m_CurrentWritePoint);

	std::vector<float>::iterator pcSrc = pData;
	for (i = 0; i < dwCopyCnt; i++, pcSrc++) {
		*WritePoint = *pcSrc;
		WritePoint++;
		if (WritePoint == m_tail) {
//td::cout << "m_bFull = true;" << std::endl;
			WritePoint = m_head;
			m_bFull = true;
		}
	}

	int comp_after = itr_comp(WritePoint, m_CurrentWritePoint);
	if((comp_before <= 0 && comp_after >= 0) || (comp_before >= 0 && comp_after <= 0) ){
		m_CurrentWritePoint = WritePoint;
	}

	// ReadPoint over-run
	if(m_bFull){
		m_CurrentReadPoint = m_CurrentWritePoint;
		if (m_CurrentReadPoint == m_tail) {
			m_CurrentReadPoint = m_head;
			
		}
	}

	return(i);
}

//-----------------------------------------------------------------------
//-----------------------------------------------------------------------
//関数オーバーロード引数がintのイテレータの場合

void copy(
		  std::vector<float>::iterator From,
		  std::vector<float>::iterator To,
		  std::vector<int>::iterator Dest)
{

	// 指定されたイテレータがバッファ内にあるようにする。
	if(distance(m_head, From) > distance(m_head, m_tail)){
		From -= m_BufferCount;
	}
	if(distance(m_head, To) > distance(m_head, m_tail)){
		To -= m_BufferCount;
	}

	// From, Toの位置関係に応じて、読み込む要素数を変更
	if(From == To){
		this->Read(Dest, From, m_BufferCount);
	}else if(From > To){
		this->Read(Dest, From, m_BufferCount - (unsigned int)std::distance(To,From)+1);			
	}else{
		this->Read(Dest, From, (unsigned int)std::distance(From,To));
	}
	return;
}


// copy の本体
unsigned int Read(	std::vector<int>::iterator pData,// 出力先
					std::vector<float>::iterator itRead,// 内部バッファの開始位置
					unsigned int dwDataCount)			// 書き出す要素数
{

	unsigned int	i;
	unsigned int	dwCopyCnt;

	if (/*pData == NULL || */dwDataCount == 0) {
		return(0);
	}
		//	受け入れ先を初期化

	// バッファサイズまでしか書かない
	if (m_BufferCount > dwDataCount){
		dwCopyCnt = dwDataCount;
	}
	else{
		dwCopyCnt = m_BufferCount;
	}

	// 書き込みバッファを追い越さないようにする
    unsigned int	dwReadable;
    if(itr_comp(itRead,m_CurrentWritePoint)>=0){
    	dwReadable = m_BufferCount - std::distance(m_CurrentWritePoint, itRead);
    }else{
    	dwReadable = std::distance(itRead, m_CurrentWritePoint);
    }

    if (dwReadable < dwCopyCnt){
        dwCopyCnt = dwReadable;
    }


	std::vector<float>::iterator ReadPoint = itRead; //読み出し位置指定イテレータ
		//	データを渡す処理
	std::vector<int>::iterator pcDst = pData;
	for (i = 0; i < dwCopyCnt; i++, pcDst++) {
		*pcDst = *ReadPoint;
		ReadPoint++;
		if (ReadPoint == m_tail) {
			ReadPoint = m_head;
			}
	}

	return(i);
}
//---------------------------------------------------------------------------
//　pData...itTo内の外部データを　内部バッファitDestへ読み込み
unsigned int insert(
			std::vector<float>::iterator itDest, 
			std::vector<int>::iterator pData, 
			std::vector<int>::iterator itTo)
{
	


	unsigned int dwDataCount = (unsigned int)std::distance(pData,itTo);//読み込み要素数
	unsigned int	i, dwCopyCnt;


	if (/*pData == NULL || */dwDataCount == 0) {
		return(0);
	}
	
	bool overrun = false;
	// バッファサイズより多いデータは末尾のみ読み込み
	if (m_BufferCount < dwDataCount){
		overrun = true;
		dwCopyCnt = m_BufferCount;
		pData = pData + dwDataCount - dwCopyCnt;
	}else{
		dwCopyCnt = dwDataCount;	
	}


	std::vector<float>::iterator WritePoint; //書き込み位置指定バッファ
	//バッファの要素に空きができないように調整
	if(distance(m_head,itDest) > m_Buffer.size()){
		WritePoint = m_CurrentWritePoint;
	}else{
		WritePoint = itDest;
	}

	// WritePointとm_CurrentWritePointの位置関係の変化を記録
	int comp_before = itr_comp(WritePoint, m_CurrentWritePoint);
	
	//	データを渡す処理
	std::vector<int>::iterator pcSrc = pData;
	for (i = 0; i < dwCopyCnt; i++, pcSrc++) {
		*WritePoint = *pcSrc;
		WritePoint++;
		if (WritePoint == m_tail) {
			WritePoint = m_head;
			m_bFull = true;
		}
	}


	// WritePointのオーバーラン処理
	int comp_after = itr_comp(WritePoint, m_CurrentWritePoint);
	if((comp_before <= 0 && comp_after >= 0) || (comp_before >= 0 && comp_after <= 0) ){
		m_CurrentWritePoint = WritePoint;
	}

	// ReadPoint のオーバーラン処理
	if(m_bFull){
		m_CurrentReadPoint = m_CurrentWritePoint;
		if (m_CurrentReadPoint == m_tail) {
			m_CurrentReadPoint = m_head;
			
		}
	}

	return(i);
}





};


#endif
