#include "BF_GHDSS.h"

#include <stdio.h>
#include <float.h>
#include <iostream>

namespace BF
{
/*// for DEBUG
void printMatrix (Matrix m){
if (m.getRowCount() == 0){
std::cout << "Rowsize=0";
return;
}
if (m.getColumnCount() == 0){
std::cout << "Columnsize=0";
return;
}
for (int i = 0 ; i < m.getRowCount() ; i++){
for (int k = 0 ; k < m.getColumnCount() ; k++){
std::cout << m.getElementAt(i,k);
}
std::cout << std::endl;
}
}*/

    GHDSS::GHDSS(int _numFreq, int _lowerfreq, int _upperfreq, int _samplingRate, double _soundSpeed, bool _muAdp, double _mu, bool _mu_ssAdp, double _mu_ss, double _scale, NlFuncType _nlFuncType, UtUpdateType _separationMatUpdateType, double _noisefloor, LcConstType _lcConstType, UsUpdateStepType _updateStepType, bool _useLcmyuAsSsmyu) :
	BF(_numFreq, _lowerfreq, _upperfreq, _samplingRate, _soundSpeed, _separationMatUpdateType),
	muAdp(_muAdp),	mu(_mu),
	mu_ssAdp(_mu_ssAdp), mu_ss(_mu_ss),
	scale(_scale),
	nlFuncType(_nlFuncType),
	noisefloor(_noisefloor),
	lcConstType(_lcConstType),
	updateStepType(_updateStepType),
	useLcmyuAsSsmyu(_useLcmyuAsSsmyu)
    {
	fprintf(stdout, "GHDSS::GHDSS(mu=%f, mu_ss=%f, scale=%f, NlFuncType=%d, separationMatUpdateType=%d, noisefloor=%f, lcConstType=%d, updateStepType=%d, useLcmyuAsSsmyu=%d",
		_mu, _mu_ss, _scale, _nlFuncType, _separationMatUpdateType, _noisefloor, _lcConstType, _updateStepType, _useLcmyuAsSsmyu);
    }
	
    GHDSS::~GHDSS()
    {
    }
    
/*== source from adaptBFv2.m
        if isempty(G) % Initialization process
            G = zeros(NSrc, NMic, NFreq);
            for nf = 1:NFreq
                for ns = 1:NSrc % for each source
                    h = conj(Adata.H(:, ns, nf)); % Target source TF
                    G(ns, :, nf) = h / NMic;
                end
            end
            return
        else % update process
            Max_val = 32768.0 ;
            Max_val = 10*log10(Max_val) ;
            MINIMUM_ZERO_VALUE = 0.0000001;
            MAX_FLOAT_VALUE = 3.0428E+038;
            ha_fThreshold = 1000;
            ftmp = (Max_val - ha_fThreshold)/10.0 ;
            Threshold_val = 10^ftmp;

            for nf = 1:NFreq % for each frequency bin
                W = G(:, :, nf);
                x = Adata.x(:, nf);
                y = Adata.y(:, nf);
                
                if min(sqrt(abs(x).^2)) < Threshold_val % 入力値チェック
                    continue;
                end
                if (nf == 1 || nf == NFreq)
                    continue;
                end

                % for SS-based calculation
                myu_ss = Adata.myu_ss;

                switch Adata.nlfunc
                    case 0 % tanh
                        ya = abs(y)+MINIMUM_ZERO_VALUE;
                        yas = Adata.scal*ya;
                        py = tanh(yas) .* (y./ya);
                        qy = (tanh(yas) + yas./(cosh(yas)).^2.*Adata.scal) .* (y./ya);
                    case 1 % x^3
                        ya = abs(y)+MINIMUM_ZERO_VALUE;
                        yas = Adata.scal*ya;
                        py = yas.^3 .* (y./ya);
                        qy = 4 * py;
                end
                Ryy = py*y';
                Ess = Ryy - diag(diag(Ryy));

                J_SS = (Ess * qy) * x';
                J_PRI = conj(J_SS);

                if myu_ss > 0 % SS_METHOD == FIX
                    myu_J_SS = myu_ss * J_SS;
                    if max(real(myu_J_SS)) > MAX_FLOAT_VALUE
%                         disp ('Data is inf!');
                    elseif max(imag(myu_J_SS)) > MAX_FLOAT_VALUE
%                         disp ('Data is inf!');
                    end
                elseif myu_ss==0 % SS_METHOD == LC_MYU
                    xx = x'*x;
                    if real(xx(1,1))^2 < MINIMUM_ZERO_VALUE
                        xx2 = 1/MINIMUM_ZERO_VALUE;
                    else
                        xx2 = 1/real(xx(1,1))^2;
                    end
                    myu_J_SS = Adata.myu * xx2 * J_SS;
                else % SS_METHOD == ADAPTIVE
                    ftmp = sum(sum(abs(Ess).^2));
                    ftmp2 = sum(sum(abs(J_SS.*J_PRI)));
                    if ftmp2 < MINIMUM_ZERO_VALUE
                        ftmp2 = ftmp2 + MINIMUM_ZERO_VALUE;
                    end
                    opt_SS_myu = ftmp/ftmp2/2;
                    myu_J_SS = opt_SS_myu * J_SS;
                    if max(real(myu_J_SS)) > MAX_FLOAT_VALUE
%                         disp ('Data is inf!');
                    elseif max(imag(myu_J_SS)) > MAX_FLOAT_VALUE
%                         disp ('Data is inf!');
                    end
                end

                yrms = sqrt(mean(abs(y).^2));
                if yrms < Adata.noisefloor
                    myu_J_SS = 0;
                end

                if Adata.updatetype == 0 % STEP
                    W = W - myu_J_SS;
                end

                % fo LC-based calculation
                D = Adata.H(:, :, nf);
                myu_gc = Adata.myu;   % LC_MYU
                Egc = W*D - eye(NSrc); % FULL
                if Adata.lc_const == 0 % DIAG
                    Egc = diag(diag(Egc));
                end
                Jpgc = Egc * D';

                if myu_gc > 0 % LC_METHOD == FIX
                    myu_J_LC = myu_gc * Jpgc;
                    if max(real(myu_J_LC)) > MAX_FLOAT_VALUE
%                         disp ('Data is inf!');
                    elseif max(imag(myu_J_LC)) > MAX_FLOAT_VALUE
%                         disp ('Data is inf!');
                    end
                else % LC_METHOD == ADAPTIVE
                    J_LCP = sum(sum(abs(Jpgc).^2)) + 1e-10;
                    if J_LCP < MINIMUM_ZERO_VALUE
                        opt_lc_myu = sum(sum(abs(Egc).^2)) / (J_LCP+MINIMUM_ZERO_VALUE) / 2;
                    else
                        opt_lc_myu = sum(sum(abs(Egc).^2)) ./ J_LCP /2;
                    end
                    myu_J_LC = opt_lc_myu * Jpgc;
                    if max(real(myu_J_LC)) > MAX_FLOAT_VALUE
%                         disp ('Data is inf!');
                    elseif max(imag(myu_J_LC)) > MAX_FLOAT_VALUE
%                         disp ('Data is inf!');
                    end
                end

                if Adata.updatetype == 0 % UPDATE == STEP
                    W = W - myu_J_LC;
                elseif Adata.updatetype == 1 % UPDATE == TOTAL
                    W = W - myu_J_LC - myu_J_SS;
                end

                G(:, :, nf) = W; % Store separation matrix of updated.

            end
            return
        end % For if initialization

*/
    void
    GHDSS::updateSources(const std::vector<SourceLocation> &sourcePositions)
    {
	int numSrc = sourcePositions.size();

	/*== 直近フレーム・分離行列データベースの音源と、このフレームでの音源との対応をとる ==*/
	std::vector<int> sourceIndices;
	std::vector<int> sourceIndicestypes;
	getSourceIndex(sourcePositions, sourceIndicestypes, sourceIndices);

	/*== 伝達関数を再計算する ==*/
	std::vector<Matrix> newHs, newGs;
	this->BF::getTransformFunctions(sourcePositions, newHs);

	for(int i = 0; i < this->numFreq; i++){
	    const Matrix &H = newHs[i];
	    int numMic = H.getRowCount();

	    Matrix G(numSrc, numMic);
	    
		// 対象周波数のみ分析
//	    if (i == 0 || i == this->numFreq-1){
	    if (i == 0 || i == this->numFreq-1 || i < lowerfreq || i > upperfreq) {
			G = Matrix::O(numSrc, numMic);
		}
		else {
		    for(int j = 0; j < numSrc; j++){
				/* 入力の j 番目の音源と同じ ID の音源が、直近フレームにあるか、データベースにあるか、見当たらないか？ */
				const int sourceIndex = sourceIndices[j];
				const int type = sourceIndicestypes[j];
				if (type == 0) { // 直近フレームにある
					// 直近フレームのデータからの引き継ぎ
					G.setRow(j, this->separationMatrices[i].getRow(sourceIndex));
				}
				else if (type == 1) { // データベースにある
					// データベースからの引き継ぎ
  					setSeparationMatricesFromDatabase(sourceIndex, j, i, G);
				}
				else { // (type == -1) 見当たらない
					// 初期値を入力
					Matrix h = H.getColumn(j);	// Hは通常の伝達関数となっている
					G.setRow(j, h.transposed()/numMic);	// 初期値はDS
				}
			}
	    }
		
		newGs.push_back(G);
	}

	/*== 分離行列データベースの座標を更新 ==*/
	updateSeparationMatricesDatabaseSource(sourcePositions, sourceIndicestypes, sourceIndices);
#if 0
int ifreqindex = 1;
std::cout << "size of newGS[" << ifreqindex << "] = (" << newGs[ifreqindex].getRowCount() << "," << newGs[ifreqindex].getColumnCount() << ")";
for (int k = 0 ; k < sourcePositions.size() ; k++){
std::cout << "BF->G(f" << ifreqindex << "s" << sourcePositions[k].id << "):";
for (int j = 0 ; j < this->micPositions.size() ; j++){
std::cout << newGs[ifreqindex].getElementAt(k, j) << ",";
}
std::cout << "@BF_GHDSS::updateSources" << std::endl;
}
#endif

	/*== 結果の格納 ==*/
	this->sourcePositions = sourcePositions;
	newHs.swap(this->transformFunctions);
	newGs.swap(this->separationMatrices);
    }
    
    void
    GHDSS::adaptSeparationMatrices()
    {
	int numSrc = this->sourcePositions.size();
	double max_val = 32768.0 ;
	max_val = 10*log10(max_val) ;
	double min_zero_value = 0.0000001;
	double threshold = 1000;
	double threshold_value = pow(10.0, (max_val - threshold)/10.0);

	std::vector<Matrix> newGs;
//	for(int i = lowerfreq; i <= upperfreq ; i++){
	for(int i = 0; i < this->numFreq; i++){
	    const Matrix &H = this->transformFunctions[i];
	    int numMic = H.getRowCount();

	    Matrix        x    = this->inputSpectrum.getColumn(i);	/* size_is(numMic, numFreq) */
	    Matrix        y    = this->outputSpectrum.getColumn(i);	/* size_is(numSrc, numFreq) */
	    Matrix        W    = this->separationMatrices[i];
	    const Matrix &D    = H;
	    
	    // 入力値チェック
	    if (x.elementwiseMultiplied(x.conjugated()).getMinimumValue().real() < threshold_value){
			newGs.push_back(W);
			continue;
		}
	    
	    // DC, ナイキストは計算しない
//	    if (i==0 || i==numFreq-1){
    	if (i == 0 || i == this->numFreq-1 || i < lowerfreq || i > upperfreq){
			newGs.push_back(W);
			continue;
	    }
		
		// 高次無相関化に基づく更新(SS-based calculation)
			// calc py, qy
		Complex v;
		double a, as;
		Matrix ryy;
		Matrix py(y.getRowCount(), 1);
		Matrix qy(y.getRowCount(), 1);
		switch(this->nlFuncType){
		case NlFuncTanh:
		{
			double tas, cas;
//if (i < 3){
//std::cout << "freq(" << i << "):";
//}
		    for(int k = 0; k < y.getRowCount(); k++){
				v = y.getElementAt(k, 0);
				a = std::abs(v) + min_zero_value;
				as = a * this->scale;
				tas = tanh(as);
				cas = cosh(as);
				py.setElementAt(k, 0, tas*(v/a));
				qy.setElementAt(k, 0, (tas + as/pow(cas, 2.0)*this->scale)*(v/a));
//if (i < 3){
//std::cout << "("<< k << ":" << tas*(v/a) << "," << (tas + as/pow(cas, 2.0)*this->scale)*(v/a) << ") ";
//}
			}
//if (i < 3){
//std::cout << std::endl;
//}
		    break;
		}
		case NlFuncCube:
		{
		    Complex as3;
		    for(int k = 0; k < y.getRowCount(); k++){
				v = y.getElementAt(k, 0);
				a = std::abs(v) + min_zero_value;
				as = a * this->scale;
				as3 = pow(as, 3.0)*(v/a);
				py.setElementAt(k, 0, as3);
				qy.setElementAt(k, 0, 4.0*as3);
		    }
		    break;
		}
		}
		ryy = py * y.transposed();
		Matrix Ess = ryy - ryy.diagonalElements().diagonalMatrix();
		Matrix J_SS = Ess * qy * x.transposed();
//std::cout << "J_SS1:";
//printMatrix(J_SS);
		Matrix J_PRI = J_SS.conjugated();

//std::cout << "ryy:";
//printMatrix(ryy);
//std::cout << "Ess:";
//printMatrix(Ess);
//std::cout << "J_SS2:";
//printMatrix(J_SS);
//std::cout << "J_PRI:";
//printMatrix(J_PRI);
		
		Matrix myu_J_SS;
		if (mu_ssAdp){
			// SS_METHOD==ADAPTIVE
			double ftmp = Ess.elementwiseMultiplied(Ess.conjugated()).getElementSum().real();
			double ftmp2 = (J_SS.elementwiseMultiplied(J_PRI)).absoluteValues().getElementSum().real();
//std::cout << "ftmp=" << ftmp << ", ftmp2=" << ftmp2 << std::endl;
			if (ftmp2 < min_zero_value){
				ftmp2 = ftmp2 + min_zero_value;
			}
			myu_J_SS = (ftmp / ftmp2 / 2.0) * J_SS;
//std::cout << "myu_J_SS:";
//printMatrix(myu_J_SS);
		}
		else {
			if (useLcmyuAsSsmyu){
				// SS_METHOD==LC_MYU
				Matrix xx = x.transposed() * x;
				double xx2;
				if (xx.getElementAt(0,0).real() < min_zero_value){
					xx2 = 1.0 / min_zero_value;
				}
				else {
					xx2 = 1.0 / pow(xx.getElementAt(0,0).real(), 2.0);
				}
				myu_J_SS = this->mu * xx2 * J_SS;
			}
			else {
				// SS_METHOD==FIX
				myu_J_SS = this->mu_ss * J_SS;
			}
		}
		
		double yrms = sqrt(y.elementwiseMultiplied(y.conjugated()).getElementSum().real() / (double)(y.getRowCount()*y.getColumnCount()));
		if (yrms < this->noisefloor){
			myu_J_SS = Matrix::O(myu_J_SS.getRowCount(), myu_J_SS.getColumnCount());
		}
		
		if (updateStepType == UsStep){
			W = W - myu_J_SS;
		}

//std::cout << "W:";
//printMatrix(W);
	    
	    
	    // 幾何制約に基づく更新(LC-based calculation)
	    Matrix	Egc  = W * D - Matrix::I(numSrc);	// FULL
	    if (lcConstType == LcDIAG){
			Egc = Egc.diagonalElements().diagonalMatrix();
		}
	    Matrix Jpgc = Egc * D.transposed();
	    Matrix myu_J_LC;
	    if (muAdp){
			// LC_METHOD==ADAPTIVE
			double J_LCP = Jpgc.elementwiseMultiplied(Jpgc.conjugated()).getElementSum().real();
			if (J_LCP < min_zero_value){
				J_LCP = J_LCP + min_zero_value;
			}
			myu_J_LC = (Egc.elementwiseMultiplied(Egc.conjugated()).getElementSum().real() / J_LCP / 2) * Jpgc;
		}
		else {
			// LC_METHOD==FIX
			myu_J_LC = this->mu * Jpgc;
		}
	    
		if (updateStepType == UsStep){
			W = W - myu_J_LC;
		}
		else if (updateStepType == UsTotal){
			W = W - myu_J_SS - myu_J_LC;
		}

//std::cout << "W:";
//printMatrix(W);
	    
	    newGs.push_back(W);
	}

	/*== 結果の格納 ==*/
	newGs.swap(this->separationMatrices);
    }   
}
