#include "BF_GSS.h"
//#include <../config.h>
#include <stdio.h>
#include <iostream>

namespace BF
{
    class InvalidStateException : public std::exception
    {
    };

    GSS::GSS(int _numFreq, int _lowerfreq, int _upperfreq, int _samplingRate, double _soundSpeed, bool _muAdp, double _mu, bool _mu_ssAdp, double _mu_ss, double _scale, NlFuncType _nlFuncType, UtUpdateType _separationMatUpdateType) :
	BF(_numFreq, _lowerfreq, _upperfreq, _samplingRate, _soundSpeed, _separationMatUpdateType),
	muAdp(_muAdp),
	mu(_mu),
	mu_ssAdp(_mu_ssAdp),
	mu_ss(_mu_ss),
	scale(_scale),
	nlFuncType(_nlFuncType)
    {
    }

    GSS::~GSS()
    {
    }

/*== source from adaptBF.m

  % For GSS, the original paper is 
  % [3] Parra, L. C., et al.: “Geometric source separation:
  % Merging convolutive source separation with geometric beamforming”,
  % IEEE Trans. SAP Vol.10, No.6, pp.352-362, 2002.
  % This program is based on instanteneous and generalized version of GSS, 
  % that is writeen in 
  % [4] H. Nakajima, et al.: "Blind Source Separation With Parameter-Free
  % Adaptive Step-Size Method for Robot Audition", 
  % IEEE Trans. ASL Vol.18, No.6, pp.1476-1485, 2010.
  % GICA and GHDSS are also included in [4].
      if isempty(G) % Initialization process
        G = zeros(NSrc, NMic, NFreq); 
        for nf = 1:NFreq
          for ns = 1:NSrc % for each source
            h = Adata.H(:, ns, nf); % Target source TF
            G(ns, :, nf) = pinv(h);
          end
        end
        return
      else % update process
        for nf = 1:NFreq % for each frequency bin
          W = G(:, :, nf);
    
          % fo GC-based calculation
          D = Adata.H(:, :, nf);
          myu_gc = Adata.myu;
          Egc = diag(diag(W*D - eye(NSrc)));
          Jpgc = Egc * D';
          
          % for SS-based calculation
          x = Adata.x(:, nf);
          y = Adata.y(:, nf);
          myu_ss = Adata.myu_ss;
          
          if method == 8 % For GSS (linear func.)
            Ryy = y*y';
          else % For ICA, HDSS (use nonlinear func.)
            switch Adata.nlfunc
            case 0 % tanh
              ya = abs(y);
              py = tanh(Adata.scal*ya) .* (y./ya);
            case 1 % x^3
              ya = abs(y);
              py = ya.^3 .* (y./ya);
            end
              Ryy = py * y';                          
          end
          Ess = Ryy - diag(diag(Ryy));
          
          if method == 8 % For GSS (linear func.)
            Jpss = 2 * Ess * W * x * x';
          elseif method == 9 % For ICA
            Jpss = Ess * W;
          elseif method == 10 % For GHDSS
            Jpss = 2 * Ess * py * x';
          end
          
          % update based on GSS, GICA, GHDSS
          W = W - myu_ss * Jpss - myu_gc * Jpgc;
          
          G(:, :, nf) = W; % Store separation matrix of updated.
          
        end
        return
      end % For if initialization
  
*/
    void
    GSS::updateSources(const std::vector<SourceLocation> &sourcePositions)
    {
	int numSrc = sourcePositions.size();

	/*== 直近フレーム・分離行列データベースの音源と、このフレームでの音源との対応をとる ==*/
	std::vector<int> sourceIndices;
	std::vector<int> sourceIndicestypes;
	getSourceIndex(sourcePositions, sourceIndicestypes, sourceIndices);	
//for(int j = 0; j < numSrc; j++)
//std::cout << "[" << j << "] sourceIndex=" << sourceIndices[j] << ", type=" << sourceIndicestypes[j] << std::endl;

	/*== 伝達関数を再計算する ==*/
	std::vector<Matrix> newHs, newGs;
	this->BF::getTransformFunctions(sourcePositions, newHs);

//	for(int i = lowerfreq; i <= upperfreq ; i++){
	for(int i = 0; i < this->numFreq; i++){
	    const Matrix &H = newHs[i];
	    int numMic = H.getRowCount();

	    Matrix G(numSrc, numMic);

		// 対象周波数のみ分析	    
//	    if (i == 0 || i == this->numFreq-1){
	    if (i == 0 || i == this->numFreq-1 || i < lowerfreq || i > upperfreq) {
			G = Matrix::O(numSrc, numMic);
		}
		else {
		    for(int j = 0; j < numSrc; j++){
				/* 入力の j 番目の音源と同じ ID の音源が、直近フレームにあるか、データベースにあるか、見当たらないか？ */
				const int sourceIndex = sourceIndices[j];
				const int type = sourceIndicestypes[j];
				if (type == 0) { // 直近フレームにある
					// 直近フレームのデータからの引き継ぎ
					G.setRow(j, this->separationMatrices[i].getRow(sourceIndex));
				}
				else if (type == 1) { // データベースにある
					// データベースからの引き継ぎ
  					setSeparationMatricesFromDatabase(sourceIndex, j, i, G);
				}
				else { // (type == -1) 見当たらない
					// 初期値を入力
					Matrix h = H.getColumn(j);	    // Hは通常の伝達関数となっている
					G.setRow(j, h.pseudoInverse());	// 初期値はDS
				}
			}
	    }
		
		newGs.push_back(G);
	}

	/*== 分離行列データベースの座標を更新 ==*/
	updateSeparationMatricesDatabaseSource(sourcePositions, sourceIndicestypes, sourceIndices);
#if 0
int ifreqindex = 1;
std::cout << "size of newGS[" << ifreqindex << "] = (" << newGs[ifreqindex].getRowCount() << "," << newGs[ifreqindex].getColumnCount() << ")";
for (int k = 0 ; k < sourcePositions.size() ; k++){
std::cout << "BF->G(f" << ifreqindex << "s" << sourcePositions[k].id << "):";
for (int j = 0 ; j < this->micPositions.size() ; j++){
std::cout << newGs[ifreqindex].getElementAt(k, j) << ",";
}
std::cout << "@BF_GSS::updateSources" << std::endl;
}
#endif

	/*== 結果の格納 ==*/
	this->sourcePositions = sourcePositions;
	newHs.swap(this->transformFunctions);
	newGs.swap(this->separationMatrices);
    }

    void
    GSS::adaptSeparationMatrices()
    {

	int numSrc = this->sourcePositions.size();

	std::vector<Matrix> newGs;
//	for(int i = lowerfreq; i <= upperfreq ; i++){
	for(int i = 0; i < this->numFreq; i++){
	    const Matrix &H = this->transformFunctions[i];
	    int numMic = H.getRowCount();

	    Matrix        W    = this->separationMatrices[i];
	    const Matrix &D    = H;
	    Matrix        Egc  = (W * D - Matrix::I(numSrc)).diagonalElements().diagonalMatrix();
	    Matrix        Jpgc = Egc * D.transposed();
	    Matrix        x    = this->inputSpectrum.getColumn(i);
	    Matrix        y    = this->outputSpectrum.getColumn(i);

	    // 対象周波数のみ分析	    
//    	if (i < lowerfreq || i > upperfreq){
    	if (i == 0 || i == this->numFreq-1 || i < lowerfreq || i > upperfreq) {
			newGs.push_back(W);
			continue;
	    }

	    // Adaptive Step Size (GSS)
	    double mu;
	    if(this->muAdp){
			if(std::abs(Jpgc.elementwiseNorm().getElementSum()) < BF::BF_EPS){
		      mu = 
		      std::abs(Egc.elementwiseNorm().getElementSum()) / 2.0 /
		      (std::abs(Jpgc.elementwiseNorm().getElementSum()) + BF::BF_EPS) *
		      this->mu;

			}else{
		      mu = 
		      std::abs(Egc.elementwiseNorm().getElementSum()) / 2.0 /
		      std::abs(Jpgc.elementwiseNorm().getElementSum()) *
		      this->mu;

			}
	    }else{
	    	  mu = this->mu;
	    }

	    Matrix ryy;
	    Matrix py(y.getRowCount(), 1);
	    switch(this->getBfAlgorithmType()){
#ifdef GSS_ENABLE
	    case BfGSS:
		ryy = y * y.transposed();
		break;
	    case BfGHDSS:
#endif
	    case BfGICA:
		{
		    switch(this->nlFuncType){
		    case NlFuncTanh:
			{
			    for(int i = 0; i < y.getRowCount(); i++){
				Complex v = y.getElementAt(i, 0);
				double  a = std::abs(v);
				if(a < BF_EPS){
				    py.setElementAt(i, 0, 0);
				}else{
					if(a < BF::BF_EPS){ 
						
						py.setElementAt(i, 0, tanh(this->scale * a) * (v / (a + BF::BF_EPS)));
					}
					else{

						if (tanh(this->scale * a )  > 1.0 )
						{py.setElementAt(i, 0, 1.0 * (v / a));}
						else{
				    		py.setElementAt(i, 0, tanh(this->scale * a) * (v / a));}
					}
				}
//				fprintf(stdout, "a=%f\n", a);
			    }
//			    py.dump(stdout, "py");
			    break;
			}

		    case NlFuncCube:
			{
			    for(int i = 0; i < y.getRowCount(); i++){
				Complex v = y.getElementAt(i, 0);
				py.setElementAt(i, 0, v * v * std::conj(v));
			    }
			    break;
			}
		    }
		    ryy = py * y.transposed();
		}
		break;

	    default:
		throw InvalidStateException();
	    }

	    Matrix Ess = ryy - ryy.diagonalElements().diagonalMatrix();
	    Matrix Jpss;
	    switch(this->getBfAlgorithmType()){
#ifdef GSS_ENABLE
	    case BfGSS:   Jpss = 2 * Ess * W * x * x.transposed(); break;
	    case BfGHDSS: Jpss = 2 * Ess * py * x.transposed(); break;
#endif
	    case BfGICA:  Jpss = Ess * W; break;
	    default:
		throw InvalidStateException();
	    }

	    // Adaptive Step Size (DS, ICA, HDSS)
	    double mu_ss;
	    Matrix den1;
	    Matrix den2;
	    if(this->mu_ssAdp){
	      switch(this->getBfAlgorithmType()){
#ifdef GSS_ENABLE
	      case BfGSS: 
		  if(std::abs(Jpss.elementwiseNorm().getElementSum()) < BF::BF_EPS){
			  mu_ss = 
				  std::abs(Ess.elementwiseNorm().getElementSum()) / 2.0 /
				  (std::abs(Jpss.elementwiseNorm().getElementSum()) + BF::BF_EPS) *
				  this->mu_ss;
		  }
		  else
		  {
			mu_ss = 
				  std::abs(Ess.elementwiseNorm().getElementSum()) / 2.0 /
				  std::abs(Jpss.elementwiseNorm().getElementSum()) *
				  this->mu_ss;
		  }
		  break;	      
	      case BfGHDSS: 
		  if(std::abs(Jpss.elementwiseNorm().getElementSum()) < BF::BF_EPS){
				mu_ss = 
				  std::abs(Ess.elementwiseNorm().getElementSum()) / 2.0 /
				  (std::abs(Jpss.elementwiseNorm().getElementSum()) + BF::BF_EPS) *
				  this->mu_ss;
		  }
		  else
		  {
				mu_ss = 
				  std::abs(Ess.elementwiseNorm().getElementSum()) / 2.0 /
				  std::abs(Jpss.elementwiseNorm().getElementSum()) *
				  this->mu_ss;
		  }
		  break;
#endif
	      case BfGICA:
		  den1 = Ess * W;
		  den2 = 2 * Ess * py * x.transposed();
		  if((den1.elementwiseFMultiplied(den2).getElementSum().real() ) < BF::BF_EPS)
		  {
				mu_ss = 
				  std::abs(Ess.elementwiseNorm().getElementSum()) / 2.0 /
				  (den1.elementwiseFMultiplied(den2).getElementSum().real() + BF::BF_EPS) * 
				  this->mu_ss; 
		  }
		  else
		  {
				mu_ss = 
				  std::abs(Ess.elementwiseNorm().getElementSum()) / 2.0 /
				  den1.elementwiseFMultiplied(den2).getElementSum().real() * 
				  this->mu_ss; 
		  }
		  break;
	      default:
		  throw InvalidStateException();
	      }
	    }else{
	      mu_ss = this->mu_ss;
	    }

	    //ess.dump(stdout, "jpss");

	    /*
	    D.dump(stdout, "D");
	    W.dump(stdout, "W");
	    x.dump(stdout, "x");
	    y.dump(stdout, "y");
	    Ess.dump(stdout, "Ess");
	    Jpss.dump(stdout, "Jpss");
	    Jpgc.dump(stdout, "Jpgc");

	    (2 * Ess).dump(stdout, "2 * Ess");
	    (2 * Ess * W).dump(stdout, "2 * Ess * W");
	    (2 * Ess * W * x).dump(stdout, "2 * Ess * w * x");
	    (2 * Ess * W * x * x.transposed()).dump(stdout, "2 * Ess * w * x * x'");
	    */

#if ENABLE_JDEBUG
	    double Jc = 
	      std::abs(Ess.elementwiseNorm().getElementSum()) + 
	      std::abs(Egc.elementwiseNorm().getElementSum());
	    double Jc1 = 
	      std::abs(Ess.elementwiseNorm().getElementSum());
	    double Jc2 = 
	      std::abs(Egc.elementwiseNorm().getElementSum());
	    std::cout << mu_ss << " " << mu << " " << Jc << " ";		
#endif

	    newGs.push_back(W - mu_ss * Jpss - mu * Jpgc);
	}

#if ENABLE_JDEBUG
	std::cout << std::endl;
#endif
	//newSeparationMatrices[0].dump(stdout, "G[freq=0]");

	/*== 結果の格納 ==*/
	newGs.swap(this->separationMatrices);
    }
}
