#include <iostream>
#include <sstream>
#include <BufferedNode.h>
#include <Buffer.h>

using namespace std;
using namespace FD;

class TextConcatenate;

DECLARE_NODE(TextConcatenate);
/*Node
 *
 * @name TextConcatenate
 * @category HARK:MISC
 * @description This concatenates string texts from arbitrary number of inputs with a specific separator. Add arbitral number of inputs to concatenate all texts.
 *
 * @input_name INPUT
 * @input_type string
 * @input_description input for an integer
 *
 * @output_name OUTPUT
 * @output_type string
 * @output_description Same as input
 * 
 * @parameter_name SEPARATOR
 * @parameter_type string
 * @parameter_value 
 * @parameter_description  Specifies separator to be used for string concatenation
 *
 * @parameter_name ENABLE_DEBUG
 * @parameter_type bool
 * @parameter_list true:false
 * @parameter_value false
 * @parameter_description enable debug print
 *
END*/

class TextConcatenate : public BufferedNode {
  int inputID;
  int outputID;
  string separator;
  bool enable_debug;

public:
  TextConcatenate(string nodeName, ParameterSet params)
    : BufferedNode(nodeName, params)
  {
    outputID = addOutput("OUTPUT");
    if(parameters.exist("SEPARATOR") == true)
      separator = object_cast<string>(parameters.get("SEPARATOR"));
    else
      separator = "";
    while (separator.find("\\t") != string::npos){
      separator.replace(separator.find("\\t"),2,"\t");
    }
    while (separator.find("\\n") != string::npos){
      separator.replace(separator.find("\\n"),2,"\n");
    }
    while (separator.find("\\v") != string::npos){
      separator.replace(separator.find("\\v"),2,"\v");
    }
    while (separator.find("\\\\") != string::npos){
      separator.replace(separator.find("\\\\"),2,"\\");
    }
    enable_debug = dereference_cast<bool>(parameters.get("ENABLE_DEBUG"));
    inOrder = true;
  }

  virtual int translateInput (const string &inputName) {
    for (unsigned int i=0; i< inputs.size(); i++) {
      if (inputs[i].name == inputName) {
	return i;
      }
    }
    return addInput(inputName);
  }

  void calculate(int output_id, int count, Buffer &out)
  {
    ostringstream os;
    for (int j = 0; j < inputs.size(); j++) {
      ObjectRef input_ptr = getInput(j, count);
      string single = object_cast<string> (input_ptr);
      os << single;
      if (j != inputs.size()-1) os << separator;
    }
    if(enable_debug) cout << os.str() << endl;
    out[count] = ObjectRef(new String(os.str()));
  }
};
